#include <stdlib.h>
#include <math.h>
#include <Time.h>
#include "Tide.h"
#include "Globals.h"

Tide::Tide()
{
	m_WaveLength=12*SECS_PER_HOUR;
	m_MaxOffset = 100;
	m_MinOffset = 0;
	m_Speed = 50;
	m_Amp = 1;
	m_CurrSpeed=m_Speed;
	m_PrevSpeed=m_CurrSpeed;
}

void Tide::Init(int Speed, int minOffset, int maxOffset)
{
	m_Speed = Speed;
	m_MaxOffset = minOffset;
	m_MinOffset = maxOffset;
}


void Tide::SetSpeed(int Speed)
{
	m_Speed = Speed;
}

void Tide::SetWaveLength(int WaveLength)
{
	m_WaveLength = WaveLength;
}

void Tide::SetOffset(int minOffset, int maxOffset)
{
	m_MaxOffset = minOffset;
	m_MinOffset = maxOffset;
}

/*
Spring tides occur during the full moon and the new moon.
At these times, the high tides are very high and the low tides are very low.

Neap tides occur during quarter moons.
The result is a smaller difference between high and low tides

MoonPhase 0-25 = Spring
MoonPhase 26-74 = Neap
MoonPhase 75-100 = Spring

So, the effect of the Moon will be a cosine wave.
*/

int Tide::CalcSpeed() {
  int minSpeed=m_Speed-(m_MaxOffset/2);
  int maxSpeed=m_Speed+(m_MaxOffset/2);
  int range = m_MaxOffset-m_MinOffset;

  double moonOffset; // tide gap between high and low
  double amplitude;  // tide

  // Calculate the gap between high and low tide based on MoonPhase()
  moonOffset=cos(((2*PI)/100)*MoonPhase());
  moonOffset=(moonOffset/2)+.5;
  moonOffset=m_MinOffset+(range*moonOffset);

  // Find out the current tidal height
  amplitude=sin(((2*PI)/m_WaveLength)*now()); 
  amplitude=(amplitude/2)+.5;

  // Adjust the calculate speed to be in our adjusted range
  m_PrevSpeed=m_CurrSpeed;
  m_CurrSpeed=minSpeed+(amplitude*moonOffset*m_Amp)+(range-(moonOffset/2));

  return m_CurrSpeed;
}

boolean Tide::isIncoming() {
  if (m_CurrSpeed-m_PrevSpeed>=0) {
    return true;
  } else {
    return false;
  }	
}
